const int buzzer = 3;  // 定义蜂鸣器的引脚
int val = 0;           // 初始化 val 变量
int led = 13;
bool youtubeOpened = false;      // 添加一个布尔变量来追踪YouTube是否已打开
bool wasAboveThreshold = false;  // 跟踪 val 是否曾经超过阈值
int delayTime = 50;
int colorStep = 5;  // 每次變色的步進
#include <Keyboard.h>
#include <Adafruit_NeoPixel.h>
#define PIN 6       // 数据引脚连接到D6
#define NUM_LEDS 5  // 使用的LED数量
Adafruit_NeoPixel strip = Adafruit_NeoPixel(NUM_LEDS, PIN, NEO_GRB + NEO_KHZ800);

void setup() {
  pinMode(buzzer, OUTPUT);  // 设置蜂鸣器引脚为输出模式
  pinMode(led, OUTPUT);     // 设置LED引脚为输出模式
  strip.begin();            // 初始化LED灯带
  strip.show();             // 初始化所有像素为关闭状态
  Serial.begin(9600);       // 初始化串口通信，波特率设为9600
  // 启动键盘功能
  Keyboard.begin();
}

void loop() {
  val = analogRead(A5);  // 读取A5引脚的模拟值
  digitalWrite(led, LOW);

  // 将读取到的值显示在串口监视器上
  Serial.print("A5 value: ");
  Serial.println(val);

  if (val > 600) {         // 当 val > 600 时，检测到磁铁
    delay(5000);          // 延迟5秒
    val = analogRead(A5);  // 重新读取 val 的值
    Serial.print("A5 value: ");
    Serial.println(val);  // 将新读取到的值显示在串口监视器上
    if (val > 600) {
      if (!youtubeOpened) {    // 如果YouTube还没有打开
        openYouTubeShort();    // 打开YouTube短片
        youtubeOpened = true;  // 设置标志，表示已经打开
        wasAboveThreshold = true;
      }
      val = analogRead(A5);  // 重新读取 val 的值
      Serial.print("A5 value: ");
      Serial.println(val);  // 将新读取到的值显示在串口监视器上
      while (val > 600) {   // 当 val > 600 时重复执行
        tone(buzzer, 988);  // 发出 988 Hz 的声音
        delay(250);         // 再次等待 500 毫秒
        noTone(buzzer);     // 关闭蜂鸣器

        digitalWrite(led, HIGH);

        // WS2812灯带显示彩虹颜色效果
        for (int i = 0; i < strip.numPixels(); i++) {
          strip.setPixelColor(i, strip.Color(255, 0, 0));  // 红色
        }
        strip.show();
        delay(250);  // 灯亮500ms
        for (int i = 0; i < strip.numPixels(); i++) {
          strip.setPixelColor(i, strip.Color(0, 0, 0));  // 关闭所有灯
        }
        strip.show();  // 灯灭500ms

        val = analogRead(A5);  // 重新读取 val 的值
        Serial.print("A5 value: ");
        Serial.println(val);  // 将新读取到的值显示在串口监视器上
      }
    }
  } else {
    if (wasAboveThreshold) {
      // 如果从超过阈值变为低于阈值，执行一次关闭操作
      closeYouTubeTab();
      wasAboveThreshold = false;  // 重置标志
    }

    gradualColorChange();

    youtubeOpened = false;  // 如果 val <= 600，重置标志
  }
}

// 辅助函数：生成彩虹颜色
uint32_t Wheel(byte WheelPos) {
  WheelPos = 255 - WheelPos;
  if (WheelPos < 85) {
    return strip.Color(255 - WheelPos * 3, 0, WheelPos * 3);
  }
  if (WheelPos < 170) {
    WheelPos -= 85;
    return strip.Color(0, WheelPos * 3, 255 - WheelPos * 3);
  }
  WheelPos -= 170;
  return strip.Color(WheelPos * 3, 255 - WheelPos * 3, 0);
}

void openYouTubeShort() {
  // 给电脑一点时间准备好
  delay(1000);
  Keyboard.press(KEY_LEFT_ALT);  // 按下左 ALT 键
  Keyboard.press(KEY_LEFT_SHIFT);
  delay(100);
  Keyboard.releaseAll();

  // 模拟按下Windows键+R来打开执行窗口
  Keyboard.press(KEY_LEFT_GUI);
  Keyboard.press('r');
  delay(100);
  Keyboard.releaseAll();
  delay(500);

  // 输入"chrome"来打开Chrome浏览器
  Keyboard.print("chrome");
  delay(500);

  // 按下Enter键
  Keyboard.press(KEY_RETURN);
  delay(500);
  Keyboard.releaseAll();
  delay(500);

  // 模拟按下Tab键
  Keyboard.press(KEY_TAB);
  delay(100);
  Keyboard.releaseAll();
  delay(500);

  // 按下Enter键
  Keyboard.press(KEY_RETURN);
  delay(100);
  Keyboard.releaseAll();
  delay(500);

  // 打开新标签页
  Keyboard.press(KEY_LEFT_CTRL);
  Keyboard.press('t');
  delay(100);
  Keyboard.releaseAll();

  // 等待新标签页打开
  delay(500);

  // 输入YouTube网址并按下Enter
  Keyboard.print("https://www.youtube.com/shorts/_ONrg-OYYmk");
  delay(500);
  Keyboard.press(KEY_RETURN);
  delay(100);
  Keyboard.releaseAll();
  delay(5000);
  Keyboard.press(KEY_TAB);
  delay(100);
  Keyboard.releaseAll();
  delay(100);
  Keyboard.press(KEY_TAB);
  delay(100);
  Keyboard.releaseAll();
  delay(100);
  Keyboard.press(KEY_TAB);
  delay(100);
  Keyboard.releaseAll();
  delay(100);
  Keyboard.press(KEY_UP_ARROW);
  delay(5000);
  Keyboard.releaseAll();
}

void closeYouTubeTab() {
  // 关闭当前的Chrome标签页
  Keyboard.press(KEY_LEFT_CTRL);
  Keyboard.press('w');
  delay(100);
  Keyboard.releaseAll();
}

void gradualColorChange() {
  int colorSteps = 255;
  int delayTime = 10;

  for (int i = 0; i < colorSteps; i++) {
    // 红到橙
    setStripColor(255, i, 0);
    delay(delayTime);
  }
  for (int i = 0; i < colorSteps; i++) {
    // 橙到黄
    setStripColor(255, 255, 0);
    delay(delayTime);
  }
  for (int i = 0; i < colorSteps; i++) {
    // 黄到绿
    setStripColor(255 - i, 255, 0);
    delay(delayTime);
  }
  for (int i = 0; i < colorSteps; i++) {
    // 绿到蓝
    setStripColor(0, 255, i);
    delay(delayTime);
  }
  for (int i = 0; i < colorSteps; i++) {
    // 蓝到靛
    setStripColor(0, 255 - i, 255);
    delay(delayTime);
  }
  for (int i = 0; i < colorSteps; i++) {
    // 靛到紫
    setStripColor(i, 0, 255);
    delay(delayTime);
  }
  for (int i = 0; i < colorSteps; i++) {
    // 紫到红
    setStripColor(255, 0, 255 - i);
    delay(delayTime);
  }
}

// 辅助函数：设置灯带颜色
void setStripColor(int r, int g, int b) {
  for (int i = 0; i < strip.numPixels(); i++) {
    strip.setPixelColor(i, strip.Color(r, g, b));
  }
  strip.show();
}